@@ -45,6 +45,7 @@
 #include "matcher/valuegepostlist.h"
 #include "net/length.h"
 #include "serialise-double.h"
+#include "stringutils.h"
 #include "termlist.h"
 
 #include "autoptr.h"
@@ -57,6 +58,7 @@
 #include <functional>
 #include <list>
 #include <string>
+#include <unordered_set>
 #include <vector>
 
 using namespace std;
@@ -1231,9 +1233,7 @@ QueryBranch::do_or_like(OrContext& ctx, QueryOptimiser * qopt, double factor,
     // QuerySynonym::done() if the single subquery is a term or MatchAll.
     Assert(subqueries.size() >= 2 || get_op() == Query::OP_SYNONYM);
 
-    vector<PostList *> postlists;
-    postlists.reserve(subqueries.size() - first);
-
+    size_t size_before = ctx.size();
     QueryVector::const_iterator q;
     for (q = subqueries.begin() + first; q != subqueries.end(); ++q) {
 	// MatchNothing subqueries should have been removed by done().
@@ -1241,9 +1241,18 @@ QueryBranch::do_or_like(OrContext& ctx, QueryOptimiser * qopt, double factor,
 	(*q).internal->postlist_sub_or_like(ctx, qopt, factor);
     }
 
-    if (elite_set_size && elite_set_size < subqueries.size()) {
-	ctx.select_elite_set(elite_set_size, subqueries.size());
-	// FIXME: not right!
+    size_t out_of = ctx.size() - size_before;
+    if (elite_set_size && elite_set_size < out_of) {
+	ctx.select_elite_set(elite_set_size, out_of);
+	// FIXME: This isn't quite right as we flatten ORs under the ELITE_SET
+	// and then pick from amongst all the subqueries.  Consider:
+	//
+	// Query subqs[] = {q1 | q2, q3 | q4};
+	// Query q(OP_ELITE_SET, begin(subqs), end(subqs), 1);
+	//
+	// Here q should be either q1 | q2 or q3 | q4, but actually it'll be
+	// just one of q1 or q2 or q3 or q4 (assuming those aren't themselves
+	// OP_OR or OP_OR-like queries).
     }
 }
 
@@ -1265,6 +1274,54 @@ QueryBranch::do_synonym(QueryOptimiser * qopt, double factor) const
     PostList * pl = ctx.postlist();
     qopt->in_synonym = old_in_synonym;
 
+    bool wdf_disjoint = false;
+    Assert(!subqueries.empty());
+    auto type = (*subqueries.begin()).get_type();
+    if (type == Query::OP_WILDCARD) {
+	// Detect common easy case where all subqueries are OP_WILDCARD whose
+	// constant prefixes form a prefix-free set.
+	wdf_disjoint = true;
+	vector<string> prefixes;
+	for (auto&& q : subqueries) {
+	    if (q.get_type() != Query::OP_WILDCARD) {
+		wdf_disjoint = false;
+		break;
+	    }
+	    auto qw = static_cast<const QueryWildcard*>(q.internal.get());
+	    prefixes.push_back(qw->get_pattern());
+	}
+
+	if (wdf_disjoint) {
+	    sort(prefixes.begin(), prefixes.end());
+	    const string* prev = nullptr;
+	    for (const auto& i : prefixes) {
+		if (prev) {
+		    if (startswith(i, *prev)) {
+			wdf_disjoint = false;
+			break;
+		    }
+		}
+		prev = &i;
+	    }
+	}
+    } else if (type == Query::LEAF_TERM) {
+	// Detect common easy case where all subqueries are terms, none of
+	// which are the same.
+	wdf_disjoint = true;
+	unordered_set<string> terms;
+	for (auto&& q : subqueries) {
+	    if (q.get_type() != Query::LEAF_TERM) {
+		wdf_disjoint = false;
+		break;
+	    }
+	    auto qt = static_cast<const QueryTerm*>(q.internal.get());
+	    if (!terms.insert(qt->get_term()).second) {
+		wdf_disjoint = false;
+		break;
+	    }
+	}
+    }
+
     // We currently assume wqf is 1 for calculating the synonym's weight
     // since conceptually the synonym is one "virtual" term.  If we were
     // to combine multiple occurrences of the same synonym expansion into
@@ -1272,9 +1329,7 @@ QueryBranch::do_synonym(QueryOptimiser * qopt, double factor) const
 
     // We build an OP_OR tree for OP_SYNONYM and then wrap it in a
     // SynonymPostList, which supplies the weights.
-    //
-    // FIXME: Detect if the subquery is wdf-disjoint?
-    RETURN(qopt->make_synonym_postlist(pl, factor, false));
+    RETURN(qopt->make_synonym_postlist(pl, factor, wdf_disjoint));
 }
 
 PostList *
