@@ -30,7 +30,7 @@
 #include "debuglog.h"
 #include "filetests.h"
 
-#ifdef XAPIAN_HAS_GLASS_BACKEND
+#if defined XAPIAN_HAS_GLASS_BACKEND && defined XAPIAN_HAS_REMOTE_BACKEND
 # include "glass/glass_databasereplicator.h"
 #endif
 #ifdef XAPIAN_HAS_CHERT_BACKEND
@@ -52,13 +52,21 @@ DatabaseReplicator::open(const string & path)
 
 #ifdef XAPIAN_HAS_CHERT_BACKEND
     if (file_exists(path + "/iamchert")) {
+# ifdef XAPIAN_HAS_REMOTE_BACKEND
 	return new ChertDatabaseReplicator(path);
+# else
+	throw FeatureUnavailableError("Replication disabled");
+# endif
     }
 #endif
 
 #ifdef XAPIAN_HAS_GLASS_BACKEND
     if (file_exists(path + "/iamglass")) {
+# ifdef XAPIAN_HAS_REMOTE_BACKEND
 	return new GlassDatabaseReplicator(path);
+# else
+	throw FeatureUnavailableError("Replication disabled");
+# endif
     }
 #endif
 
