@@ -359,6 +359,7 @@ GlassValueManager::add_document(Xapian::docid did, const Xapian::Document &doc,
     while (it != doc.values_end()) {
 	Xapian::valueno slot = it.get_valueno();
 	string value = *it;
+	Assert(!value.empty());
 
 	// Update the statistics.
 	std::pair<map<Xapian::valueno, ValueStats>::iterator, bool> i;
@@ -551,6 +552,13 @@ GlassValueManager::get_value_stats(Xapian::valueno slot, ValueStats & stats) con
 	    if (pos == 0) throw Xapian::DatabaseCorruptError("Incomplete stats item in value table");
 	    throw Xapian::RangeError("Lower bound in value table is too large");
 	}
+	if (stats.lower_bound.empty() && stats.freq != 0) {
+	    Assert(false);
+	    // This shouldn't happen, but let's code defensively and set the
+	    // smallest valid lower bound (a single zero byte) if it somehow
+	    // does (it's been reported by a notmuch user).
+	    stats.lower_bound.assign(1, '\0');
+	}
 	size_t len = end - pos;
 	if (len == 0) {
 	    stats.upper_bound = stats.lower_bound;
