@@ -1,7 +1,7 @@
 /** @file slowvaluelist.cc
  * @brief Slow implementation for backends which don't streamed values.
  */
-/* Copyright (C) 2008,2014 Olly Betts
+/* Copyright (C) 2008,2011,2013,2014,2018 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,6 +24,7 @@
 
 #include "document.h"
 #include "str.h"
+#include "unicode/description_append.h"
 
 #include "xapian/error.h"
 
@@ -65,9 +66,11 @@ SlowValueList::next()
 	try {
 	    // Open document lazily so that we don't waste time checking for
 	    // its existence.
+	    void * d = db->open_document(current_did, true);
+	    if (!d)
+		continue;
 	    AutoPtr<Xapian::Document::Internal>
-		doc(db->open_document(current_did, true));
-	    if (!doc.get()) continue;
+		doc(static_cast<Xapian::Document::Internal*>(d));
 	    string value = doc->get_value(slot);
 	    if (!value.empty()) {
 		swap(current_value, value);
@@ -85,13 +88,6 @@ void
 SlowValueList::skip_to(Xapian::docid did)
 {
     if (did <= current_did) return;
-
-    if (did > last_docid) {
-	// Indicate that we're at_end().
-	last_docid = 0;
-	return;
-    }
-
     current_did = did - 1;
     next();
 }
@@ -99,7 +95,9 @@ SlowValueList::skip_to(Xapian::docid did)
 bool
 SlowValueList::check(Xapian::docid did)
 {
-    if (did <= current_did) return true;
+    if (did <= current_did) {
+	return !current_value.empty();
+    }
 
     if (did > last_docid) {
 	// Indicate that we're at_end().
@@ -109,14 +107,16 @@ SlowValueList::check(Xapian::docid did)
 
     current_did = did;
     try {
-	AutoPtr<Xapian::Document::Internal>
-	doc(db->open_document(current_did, true));
-	if (doc.get()) {
+	void * d = db->open_document(current_did, true);
+	if (d) {
+	    AutoPtr<Xapian::Document::Internal>
+		doc(static_cast<Xapian::Document::Internal*>(d));
 	    current_value = doc->get_value(slot);
 	    if (!current_value.empty()) return true;
 	}
     } catch (const Xapian::DocNotFoundError &) {
     }
+    current_value = string();
     return false;
 }
 
@@ -129,7 +129,7 @@ SlowValueList::get_description() const
 	desc += ", docid=";
 	desc += str(current_did);
 	desc += ", value=\"";
-	desc += current_value;
+	description_append(desc, current_value);
 	desc += "\")";
     } else {
 	desc += ", atend)";
