@@ -31,9 +31,9 @@
 
 #include "gnu_getopt.h"
 
+#include <cerrno>
 #include <cstring>
 #include <cstdlib>
-#include "safeerrno.h"
 #include "unicode/description_append.h"
 
 #include "unicode/description_append.cc"
@@ -121,7 +121,7 @@ show_db_stats(Database &db)
 
     if (count_zero_length_docs) {
 	Xapian::doccount empty_docs = 0;
-	if (db.get_avlength() == 0) {
+	if (db.get_total_length() == 0) {
 	    // All documents are empty.
 	    empty_docs = db.get_doccount();
 	} else {
@@ -290,8 +290,6 @@ show_termlists(Database &db,
     }
 }
 
-static Stem stemmer;
-
 int
 main(int argc, char **argv) try {
     if (argc > 1 && argv[1][0] == '-') {
@@ -310,6 +308,7 @@ main(int argc, char **argv) try {
     vector<docid> recnos;
     vector<string> terms;
     vector<string> dbs;
+    Stem stemmer;
 
     valueno slot = 0; // Avoid "may be used uninitialised" warnings.
     bool slot_set = false;
