@@ -25,6 +25,10 @@
 #ifndef XAPIAN_INCLUDED_WORDACCESS_H
 #define XAPIAN_INCLUDED_WORDACCESS_H
 
+#ifndef PACKAGE
+# error config.h must be included first in each C++ source file
+#endif
+
 #include <cstdint>
 #include <type_traits>
 
