@@ -166,6 +166,9 @@ const char * dummy[] = {
 "/// Base (signed) type for Xapian::termcount and related types.",
 "#define XAPIAN_TERMCOUNT_BASE_TYPE ", XAPIAN_TERMCOUNT_BASE_TYPE,
 "",
+"/// Base (signed) type for Xapian::termpos.",
+"#define XAPIAN_TERMPOS_BASE_TYPE ", XAPIAN_TERMPOS_BASE_TYPE,
+"",
 "/// Type for returning total document length.",
 "#define XAPIAN_TOTALLENGTH_TYPE ", XAPIAN_REVISION_TYPE,
 "",
