@@ -1,12 +1,14 @@
+#include <ctype.h>   /* for toupper etc */
 #include <stdio.h>   /* for fprintf etc */
 #include <stdlib.h>  /* for free etc */
 #include <string.h>  /* for strcmp */
 #include "header.h"
 
-#define DEFAULT_PACKAGE "org.tartarus.snowball.ext"
-#define DEFAULT_BASE_CLASS "org.tartarus.snowball.SnowballProgram"
-#define DEFAULT_AMONG_CLASS "org.tartarus.snowball.Among"
-#define DEFAULT_STRING_CLASS "java.lang.StringBuilder"
+#define DEFAULT_JAVA_PACKAGE "org.tartarus.snowball.ext"
+#define DEFAULT_JAVA_BASE_CLASS "org.tartarus.snowball.SnowballProgram"
+#define DEFAULT_JAVA_AMONG_CLASS "org.tartarus.snowball.Among"
+#define DEFAULT_JAVA_STRING_CLASS "java.lang.StringBuilder"
+
 #define DEFAULT_GO_PACKAGE "snowball"
 #define DEFAULT_GO_SNOWBALL_RUNTIME "github.com/snowballstem/snowball/go"
 
@@ -15,6 +17,10 @@
 #define DEFAULT_CS_AMONG_CLASS "Among"
 #define DEFAULT_CS_STRING_CLASS "StringBuilder"
 
+#define DEFAULT_JS_BASE_CLASS "BaseStemmer"
+
+#define DEFAULT_PYTHON_BASE_CLASS "BaseStemmer"
+
 static int eq(const char * s1, const char * s2) {
     return strcmp(s1, s2) == 0;
 }
@@ -30,6 +36,9 @@ static void print_arglist(void) {
                     "             [-cs[harp]]\n"
 #endif
                     "             [-c++]\n"
+#ifndef DISABLE_PASCAL
+                    "             [-pascal]\n"
+#endif
 #ifndef DISABLE_PYTHON
                     "             [-py[thon]]\n"
 #endif
@@ -85,6 +94,10 @@ static int read_options(struct options * o, int argc, char * argv[]) {
     char * s;
     int i = 1;
     int new_argc = 1;
+    /* Note down the last option used to specify an explicit encoding so
+     * we can warn we ignored it for languages with a fixed encoding.
+     */
+    const char * encoding_opt = NULL;
 
     /* set defaults: */
 
@@ -93,11 +106,10 @@ static int read_options(struct options * o, int argc, char * argv[]) {
     o->externals_prefix = NULL;
     o->variables_prefix = 0;
     o->runtime_path = 0;
-    o->parent_class_name = DEFAULT_BASE_CLASS;
-    o->string_class = DEFAULT_STRING_CLASS;
-    o->among_class = DEFAULT_AMONG_CLASS;
-    o->package = DEFAULT_PACKAGE;
-    o->go_package = DEFAULT_GO_PACKAGE;
+    o->parent_class_name = NULL;
+    o->string_class = NULL;
+    o->among_class = NULL;
+    o->package = NULL;
     o->go_snowball_runtime = DEFAULT_GO_SNOWBALL_RUNTIME;
     o->name = NULL;
     o->make_lang = LANG_C;
@@ -129,39 +141,30 @@ static int read_options(struct options * o, int argc, char * argv[]) {
 #ifndef DISABLE_JS
             if (eq(s, "-js")) {
                 o->make_lang = LANG_JAVASCRIPT;
-                o->encoding = ENC_WIDECHARS;
                 continue;
             }
 #endif
 #ifndef DISABLE_RUST
             if (eq(s, "-rust")) {
                 o->make_lang = LANG_RUST;
-                o->encoding = ENC_UTF8;
                 continue;
             }
 #endif
 #ifndef DISABLE_GO
             if (eq(s, "-go")) {
                 o->make_lang = LANG_GO;
-                o->encoding = ENC_UTF8;
                 continue;
             }
 #endif
 #ifndef DISABLE_JAVA
             if (eq(s, "-j") || eq(s, "-java")) {
                 o->make_lang = LANG_JAVA;
-                o->encoding = ENC_WIDECHARS;
                 continue;
             }
 #endif
 #ifndef DISABLE_CSHARP
             if (eq(s, "-cs") || eq(s, "-csharp")) {
                 o->make_lang = LANG_CSHARP;
-		o->encoding = ENC_WIDECHARS;
-                o->parent_class_name = DEFAULT_CS_BASE_CLASS;
-                o->string_class = DEFAULT_CS_STRING_CLASS;
-                o->among_class = DEFAULT_CS_AMONG_CLASS;
-                o->package = DEFAULT_CS_NAMESPACE;
                 continue;
             }
 #endif
@@ -169,14 +172,20 @@ static int read_options(struct options * o, int argc, char * argv[]) {
                 o->make_lang = LANG_CPLUSPLUS;
                 continue;
             }
+#ifndef DISABLE_PASCAL
+            if (eq(s, "-pascal")) {
+                o->make_lang = LANG_PASCAL;
+                continue;
+            }
+#endif
 #ifndef DISABLE_PYTHON
             if (eq(s, "-py") || eq(s, "-python")) {
                 o->make_lang = LANG_PYTHON;
-                o->encoding = ENC_WIDECHARS;
                 continue;
             }
 #endif
             if (eq(s, "-w") || eq(s, "-widechars")) {
+                encoding_opt = s;
                 o->encoding = ENC_WIDECHARS;
                 continue;
             }
@@ -215,6 +224,7 @@ static int read_options(struct options * o, int argc, char * argv[]) {
                 continue;
             }
             if (eq(s, "-u") || eq(s, "-utf8")) {
+                encoding_opt = s;
                 o->encoding = ENC_UTF8;
                 continue;
             }
@@ -243,7 +253,7 @@ static int read_options(struct options * o, int argc, char * argv[]) {
 #ifndef DISABLE_GO
             if (eq(s, "-gop") || eq(s, "-gopackage")) {
                 check_lim(i, argc);
-                o->go_package = argv[i++];
+                o->package = argv[i++];
                 continue;
             }
             if (eq(s, "-gor") || eq(s, "-goruntime")) {
@@ -262,6 +272,61 @@ static int read_options(struct options * o, int argc, char * argv[]) {
     }
     argv[new_argc] = NULL;
 
+    /* Set language-dependent defaults. */
+    switch (o->make_lang) {
+        case LANG_C:
+        case LANG_CPLUSPLUS:
+            encoding_opt = NULL;
+            break;
+        case LANG_CSHARP:
+            o->encoding = ENC_WIDECHARS;
+            if (!o->parent_class_name)
+                o->parent_class_name = DEFAULT_CS_BASE_CLASS;
+            if (!o->string_class)
+                o->string_class = DEFAULT_CS_STRING_CLASS;
+            if (!o->among_class)
+                o->among_class = DEFAULT_CS_AMONG_CLASS;
+            if (!o->package)
+                o->package = DEFAULT_CS_NAMESPACE;
+            break;
+        case LANG_GO:
+            o->encoding = ENC_UTF8;
+            if (!o->package)
+                o->package = DEFAULT_GO_PACKAGE;
+            break;
+        case LANG_JAVA:
+            o->encoding = ENC_WIDECHARS;
+            if (!o->parent_class_name)
+                o->parent_class_name = DEFAULT_JAVA_BASE_CLASS;
+            if (!o->string_class)
+                o->string_class = DEFAULT_JAVA_STRING_CLASS;
+            if (!o->among_class)
+                o->among_class = DEFAULT_JAVA_AMONG_CLASS;
+            if (!o->package)
+                o->package = DEFAULT_JAVA_PACKAGE;
+            break;
+        case LANG_JAVASCRIPT:
+            o->encoding = ENC_WIDECHARS;
+            if (!o->parent_class_name)
+                o->parent_class_name = DEFAULT_JS_BASE_CLASS;
+            break;
+        case LANG_PYTHON:
+            o->encoding = ENC_WIDECHARS;
+            if (!o->parent_class_name)
+                o->parent_class_name = DEFAULT_PYTHON_BASE_CLASS;
+            break;
+        case LANG_RUST:
+            o->encoding = ENC_UTF8;
+            break;
+        default:
+            break;
+    }
+
+    if (encoding_opt) {
+        fprintf(stderr, "warning: %s only meaningful for C and C++\n",
+                encoding_opt);
+    }
+
     if (o->make_lang != LANG_C && o->make_lang != LANG_CPLUSPLUS) {
 	if (o->runtime_path) {
 	    fprintf(stderr, "warning: -r/-runtime only meaningful for C and C++\n");
@@ -292,6 +357,7 @@ static int read_options(struct options * o, int argc, char * argv[]) {
             char * new_name = malloc(len + 1);
             switch (o->make_lang) {
                 case LANG_CSHARP:
+                case LANG_PASCAL:
                     /* Upper case initial letter. */
                     memcpy(new_name, leaf, len);
                     new_name[0] = toupper(new_name[0]);
@@ -302,7 +368,7 @@ static int read_options(struct options * o, int argc, char * argv[]) {
                      * underscore+letter or hyphen+letter to an upper case
                      * letter.
                      */
-                    int i, j = 0;
+                    size_t i, j = 0;
                     int uc_next = true;
                     for (i = 0; i != len; ++i) {
 			unsigned char ch = leaf[i];
@@ -340,12 +406,10 @@ extern int main(int argc, char * argv[]) {
     NEW(options, o);
     argc = read_options(o, argc, argv);
     {
-        symbol * filename = add_s_to_b(0, argv[1]);
-        char * file;
-        symbol * u = get_input(filename, &file);
-        lose_b(filename);
+        char * file = argv[1];
+        symbol * u = get_input(file);
         if (u == 0) {
-            fprintf(stderr, "Can't open input %s\n", argv[1]);
+            fprintf(stderr, "Can't open input %s\n", file);
             exit(1);
         }
         {
@@ -359,16 +423,16 @@ extern int main(int argc, char * argv[]) {
              * 'get' uses. */
             for (i = 2; i != argc; ++i) {
                 NEW(input, q);
-                filename = add_s_to_b(0, argv[i]);
-                u = get_input(filename, &file);
-                lose_b(filename);
+                file = argv[i];
+                u = get_input(file);
                 if (u == 0) {
-                    fprintf(stderr, "Can't open input %s\n", argv[i]);
+                    fprintf(stderr, "Can't open input %s\n", file);
                     exit(1);
                 }
                 q->p = u;
                 q->c = 0;
                 q->file = file;
+                q->file_needs_freeing = false;
                 q->line_number = 1;
                 *next_input_ptr = q;
                 next_input_ptr = &(q->next);
@@ -413,6 +477,16 @@ extern int main(int argc, char * argv[]) {
                     fclose(o->output_src);
                 }
 #endif
+#ifndef DISABLE_PASCAL
+                if (o->make_lang == LANG_PASCAL) {
+                    symbol *b = add_s_to_b(0, s);
+                    b = add_s_to_b(b, ".pas");
+                    o->output_src = get_output(b);
+                    lose_b(b);
+                    generate_program_pascal(g);
+                    fclose(o->output_src);
+                }
+#endif
 #ifndef DISABLE_PYTHON
                 if (o->make_lang == LANG_PYTHON) {
                     symbol * b = add_s_to_b(0, s);
