@@ -67,9 +67,13 @@
 
 using namespace std;
 
+namespace Xapian {
+
 #define CREATE_SIZE 16
 
-extern symbol * create_s() {
+symbol *
+SnowballStemImplementation::create_s()
+{
     void * mem = malloc(HEAD + (CREATE_SIZE + 1) * sizeof(symbol));
     if (mem == NULL) throw std::bad_alloc();
     symbol * p = reinterpret_cast<symbol*>(HEAD + static_cast<char *>(mem));
@@ -86,7 +90,9 @@ extern symbol * create_s() {
    -- used to implement hop and next in the utf8 case.
 */
 
-extern int skip_utf8(const symbol * p, int c, int lb, int l, int n) {
+int
+SnowballStemImplementation::skip_utf8(const symbol * p, int c, int lb, int l, int n)
+{
     if (n >= 0) {
 	for (; n > 0; --n) {
 	    if (c >= l) return -1;
@@ -116,7 +122,9 @@ extern int skip_utf8(const symbol * p, int c, int lb, int l, int n) {
 /* Increase the size of the buffer pointed to by p to at least n symbols.
  * If insufficient memory, throw std::bad_alloc().
  */
-static symbol * increase_size(symbol * p, int n) {
+symbol *
+SnowballStemImplementation::increase_size(symbol * p, int n)
+{
     int new_size = n + 20;
     void * mem = realloc(reinterpret_cast<char *>(p) - HEAD,
 			 HEAD + (new_size + 1) * sizeof(symbol));
@@ -128,7 +136,6 @@ static symbol * increase_size(symbol * p, int n) {
     return q;
 }
 
-namespace Xapian {
 
 StemImplementation::~StemImplementation() { }
 
