@@ -1,7 +1,7 @@
 /** @file msetcmp.h
- * @brief MSetItem comparison functions and functors.
+ * @brief MSetItem comparison functions.
  */
-/* Copyright (C) 2006,2007,2011 Olly Betts
+/* Copyright (C) 2006,2007,2011,2017 Olly Betts
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,29 +19,16 @@
  * USA
  */
 
-
 #ifndef XAPIAN_INCLUDED_MSETCMP_H
 #define XAPIAN_INCLUDED_MSETCMP_H
 
 #include "api/omenquireinternal.h"
 
 // typedef for MSetItem comparison function.
-typedef bool (* mset_cmp)(const Xapian::Internal::MSetItem &,
-			  const Xapian::Internal::MSetItem &);
+typedef bool (* MSetCmp)(const Xapian::Internal::MSetItem &,
+			 const Xapian::Internal::MSetItem &);
 
 /// Select the appropriate msetcmp function.
-mset_cmp get_msetcmp_function(Xapian::Enquire::Internal::sort_setting sort_by, bool sort_forward, bool sort_value_forward);
-
-/// MSetItem comparison functor.
-class MSetCmp {
-    mset_cmp fn;
-  public:
-    explicit MSetCmp(mset_cmp fn_) : fn(fn_) { }
-    /// Return true if MSetItem a should be ranked above MSetItem b.
-    bool operator()(const Xapian::Internal::MSetItem &a,
-		    const Xapian::Internal::MSetItem &b) const {
-	return fn(a, b);
-    }
-};
+MSetCmp get_msetcmp_function(Xapian::Enquire::Internal::sort_setting sort_by, bool sort_forward, bool sort_value_forward);
 
 #endif // XAPIAN_INCLUDED_MSETCMP_H
