@@ -1,7 +1,7 @@
 /** @file termgenerator_internal.cc
  * @brief TermGenerator class internals
  */
-/* Copyright (C) 2007,2010,2011,2012,2015,2016,2017,2018 Olly Betts
+/* Copyright (C) 2007,2010,2011,2012,2015,2016,2017,2018,2019 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -177,12 +177,16 @@ parse_terms(Utf8Iterator itor, bool cjk_ngram, bool with_positions, ACTION actio
 	    if (cjk_ngram &&
 		CJK::codepoint_is_cjk(*itor) &&
 		Unicode::is_wordchar(*itor)) {
-		const string & cjk = CJK::get_cjk(itor);
-		for (CJKTokenIterator tk(cjk); tk != CJKTokenIterator(); ++tk) {
-		    const string & cjk_token = *tk;
-		    if (!action(cjk_token, with_positions && tk.get_length() == 1, itor))
+		CJKTokenIterator tk(itor);
+		while (tk != CJKTokenIterator()) {
+		    const string& cjk_token = *tk;
+		    if (!action(cjk_token, with_positions && tk.unigram(),
+				tk.get_utf8iterator()))
 			return;
+		    ++tk;
 		}
+		// Update itor to end of CJK text span.
+		itor = tk.get_utf8iterator();
 		while (true) {
 		    if (itor == Utf8Iterator()) return;
 		    ch = check_wordchar(*itor);
@@ -707,7 +711,10 @@ MSet::Internal::snippet(const string & text,
 	return text;
     }
 
-    bool cjk_ngram = CJK::is_cjk_enabled();
+    bool cjk_ngram = (flags & MSet::SNIPPET_CJK_NGRAM);
+    if (!cjk_ngram) {
+	cjk_ngram = CJK::is_cjk_enabled();
+    }
 
     size_t term_start = 0;
     double min_tw = 0, max_tw = 0;
