@@ -70,8 +70,7 @@ DEFINE_TESTCASE(termstats, backend) {
 }
 
 // Check that stub databases work.
-DEFINE_TESTCASE(stubdb1, backend && !inmemory && !remote) {
-    // Only works for backends which have a path.
+DEFINE_TESTCASE(stubdb1, path) {
     mkdir(".stub", 0755);
     const char * dbpath = ".stub/stubdb1";
     ofstream out(dbpath);
@@ -96,8 +95,7 @@ DEFINE_TESTCASE(stubdb1, backend && !inmemory && !remote) {
 }
 
 // Check that stub databases work remotely.
-DEFINE_TESTCASE(stubdb2, backend && !inmemory && !remote) {
-    // Only works for backends which have a path.
+DEFINE_TESTCASE(stubdb2, path) {
     mkdir(".stub", 0755);
     const char * dbpath = ".stub/stubdb2";
     ofstream out(dbpath);
@@ -226,8 +224,7 @@ DEFINE_TESTCASE(stubdb2, backend && !inmemory && !remote) {
 }
 
 // Regression test - bad entries were ignored after a good entry prior to 1.0.8.
-DEFINE_TESTCASE(stubdb3, backend && !inmemory && !remote) {
-    // Only works for backends which have a path.
+DEFINE_TESTCASE(stubdb3, path) {
     mkdir(".stub", 0755);
     const char * dbpath = ".stub/stubdb3";
     ofstream out(dbpath);
@@ -246,8 +243,7 @@ DEFINE_TESTCASE(stubdb3, backend && !inmemory && !remote) {
 }
 
 // Test a stub database with just a bad entry.
-DEFINE_TESTCASE(stubdb4, backend && !inmemory && !remote) {
-    // Only works for backends which have a path.
+DEFINE_TESTCASE(stubdb4, !backend) {
     mkdir(".stub", 0755);
     const char * dbpath = ".stub/stubdb4";
     ofstream out(dbpath);
@@ -266,8 +262,7 @@ DEFINE_TESTCASE(stubdb4, backend && !inmemory && !remote) {
 
 // Test a stub database with a bad entry with no spaces (prior to 1.1.0 this
 // was deliberately allowed, though not documented.
-DEFINE_TESTCASE(stubdb5, backend && !inmemory && !remote) {
-    // Only works for backends which have a path.
+DEFINE_TESTCASE(stubdb5, path) {
     mkdir(".stub", 0755);
     const char * dbpath = ".stub/stubdb5";
     ofstream out(dbpath);
@@ -1587,27 +1582,27 @@ DEFINE_TESTCASE(consistency1, backend && !remote) {
 }
 
 // tests that specifying a nonexistent input file throws an exception.
-DEFINE_TESTCASE(chertdatabaseopeningerror1, chert) {
+DEFINE_TESTCASE(chertdatabasenotfounderror1, chert) {
 #ifdef XAPIAN_HAS_CHERT_BACKEND
     mkdir(".chert", 0755);
 
-    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
+    TEST_EXCEPTION(Xapian::DatabaseNotFoundError,
 	    Xapian::Database(".chert/nosuchdirectory",
 		Xapian::DB_BACKEND_CHERT));
-    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
+    TEST_EXCEPTION(Xapian::DatabaseNotFoundError,
 	    Xapian::WritableDatabase(".chert/nosuchdirectory",
 		Xapian::DB_OPEN|Xapian::DB_BACKEND_CHERT));
 
     mkdir(".chert/emptydirectory", 0700);
-    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
+    TEST_EXCEPTION(Xapian::DatabaseNotFoundError,
 	    Xapian::Database(".chert/emptydirectory",
 		Xapian::DB_BACKEND_CHERT));
 
     touch(".chert/somefile");
-    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
+    TEST_EXCEPTION(Xapian::DatabaseNotFoundError,
 	    Xapian::Database(".chert/somefile",
 		Xapian::DB_BACKEND_CHERT));
-    TEST_EXCEPTION(Xapian::DatabaseOpeningError,
+    TEST_EXCEPTION(Xapian::DatabaseNotFoundError,
 	    Xapian::WritableDatabase(".chert/somefile",
 		Xapian::DB_OPEN|Xapian::DB_BACKEND_CHERT));
     TEST_EXCEPTION(Xapian::DatabaseCreateError,
@@ -1624,6 +1619,43 @@ DEFINE_TESTCASE(chertdatabaseopeningerror1, chert) {
     return true;
 }
 
+DEFINE_TESTCASE(glassdatabasenotfounderror1, glass) {
+#ifdef XAPIAN_HAS_GLASS_BACKEND
+    mkdir(".glass", 0755);
+
+    TEST_EXCEPTION(Xapian::DatabaseNotFoundError,
+	    Xapian::Database(".glass/nosuchdirectory",
+		Xapian::DB_BACKEND_GLASS));
+    TEST_EXCEPTION(Xapian::DatabaseNotFoundError,
+	    Xapian::WritableDatabase(".glass/nosuchdirectory",
+		Xapian::DB_OPEN|Xapian::DB_BACKEND_GLASS));
+
+    mkdir(".glass/emptydirectory", 0700);
+    TEST_EXCEPTION(Xapian::DatabaseNotFoundError,
+	    Xapian::Database(".glass/emptydirectory",
+		Xapian::DB_BACKEND_GLASS));
+
+    touch(".glass/somefile");
+    TEST_EXCEPTION(Xapian::DatabaseNotFoundError,
+	    Xapian::Database(".glass/somefile",
+		Xapian::DB_BACKEND_GLASS));
+    TEST_EXCEPTION(Xapian::DatabaseNotFoundError,
+	    Xapian::WritableDatabase(".glass/somefile",
+		Xapian::DB_OPEN|Xapian::DB_BACKEND_GLASS));
+    TEST_EXCEPTION(Xapian::DatabaseCreateError,
+	    Xapian::WritableDatabase(".glass/somefile",
+		Xapian::DB_CREATE|Xapian::DB_BACKEND_GLASS));
+    TEST_EXCEPTION(Xapian::DatabaseCreateError,
+	    Xapian::WritableDatabase(".glass/somefile",
+		Xapian::DB_CREATE_OR_OPEN|Xapian::DB_BACKEND_GLASS));
+    TEST_EXCEPTION(Xapian::DatabaseCreateError,
+	    Xapian::WritableDatabase(".glass/somefile",
+		Xapian::DB_CREATE_OR_OVERWRITE|Xapian::DB_BACKEND_GLASS));
+#endif
+
+    return true;
+}
+
 /// Test opening of a chert database
 DEFINE_TESTCASE(chertdatabaseopen1, chert) {
 #ifdef XAPIAN_HAS_CHERT_BACKEND
