@@ -1,7 +1,7 @@
 /** @file backendmanager_multi.cc
  * @brief BackendManager subclass for multi databases.
  */
-/* Copyright (C) 2007,2008,2009,2011,2012,2013,2015,2017 Olly Betts
+/* Copyright (C) 2007,2008,2009,2011,2012,2013,2015,2017,2019 Olly Betts
  * Copyright (C) 2008 Lemur Consulting Ltd
  *
  * This program is free software; you can redistribute it and/or
@@ -23,6 +23,7 @@
 
 #include "backendmanager_multi.h"
 
+#include "errno_to_string.h"
 #include "filetests.h"
 #include "index_utils.h"
 #include "str.h"
@@ -87,7 +88,7 @@ BackendManagerMulti::createdb_multi(const string& name,
 	string msg = "Couldn't create file '";
 	msg += tmpfile;
 	msg += "' (";
-	msg += strerror(errno);
+	errno_to_string(errno, msg);
 	msg += ')';
 	throw msg;
     }
@@ -157,6 +158,12 @@ BackendManagerMulti::get_writable_database_as_database()
     return Xapian::Database(last_wdb_path);
 }
 
+string
+BackendManagerMulti::get_generated_database_path(const string& name)
+{
+    return BackendManagerMulti::get_writable_database_path(name);
+}
+
 Xapian::WritableDatabase
 BackendManagerMulti::get_writable_database_again()
 {
