@@ -48,6 +48,7 @@
 # include <cstdlib> // For free().
 #endif
 
+#include "errno_to_string.h"
 #include "noreturn.h"
 #include "str.h"
 
@@ -122,7 +123,7 @@ try_next_port:
     int fds[2];
     if (socketpair(AF_UNIX, SOCK_STREAM|SOCK_CLOEXEC, PF_UNSPEC, fds) < 0) {
 	string msg("Couldn't create socketpair: ");
-	msg += strerror(errno);
+	errno_to_string(errno, msg);
 	throw msg;
     }
 
@@ -154,7 +155,7 @@ try_next_port:
 	int fork_errno = errno;
 	close(fds[0]);
 	string msg("Couldn't fork: ");
-	msg += strerror(fork_errno);
+	errno_to_string(fork_errno, msg);
 	throw msg;
     }
 
@@ -166,7 +167,7 @@ try_next_port:
 	string msg("Failed to run command '");
 	msg += cmd;
 	msg += "': ";
-	msg += strerror(errno);
+	errno_to_string(errno, msg);
 	throw msg;
     }
 
@@ -179,7 +180,7 @@ try_next_port:
 	    int status;
 	    if (waitpid(child, &status, 0) == -1) {
 		string msg("waitpid failed: ");
-		msg += strerror(errno);
+		errno_to_string(errno, msg);
 		throw msg;
 	    }
 	    if (++port < 65536 && status != 0) {
