@@ -79,6 +79,8 @@ class TestRunner {
 	CHERT		= 0x00004000,
 	GLASS		= 0x00008000,
 	COMPACT		= 0x00010000,
+	/// Requires get_database_path() or similar.
+	PATH		= 0x00020000,
     };
 
   public:
