@@ -74,19 +74,23 @@ void test_mset_order_equal(const Xapian::MSet &mset1,
 	(M).size() << "' expected '" << (S) << "':\n" << \
 	"Full mset was:\n" << (M))
 
-/// Check that a piece of code throws an expected exception.
-#define TEST_EXCEPTION(a,b) do {\
-	expected_exception = STRINGIZE(a);\
-	if (strncmp(expected_exception, "Xapian::", 8) == 0)\
-	    expected_exception += 8;\
-	if (verbose)\
-	    tout << "Expecting exception " << expected_exception << endl;\
-	try { b; FAIL_TEST(TESTCASE_LOCN(Expected #a)); }\
-	catch (const a &e) {\
-	    if (verbose)\
-		tout << "Caught expected " << expected_exception\
-		     << " exception: " << e.get_description() << endl;\
-	}\
+/// Check that CODE throws exactly Xapian exception TYPE.
+#define TEST_EXCEPTION(TYPE, CODE) \
+    do { \
+	expected_exception = STRINGIZE(TYPE); \
+	if (strncmp(expected_exception, "Xapian::", \
+		    CONST_STRLEN("Xapian::")) == 0) { \
+	    expected_exception += CONST_STRLEN("Xapian::"); \
+	} \
+	try { \
+	    CODE; \
+	    FAIL_TEST("Expected " << expected_exception << " not thrown"); \
+	} catch (const TYPE& e) { \
+	    if (strcmp(expected_exception, e.get_type()) != 0) { \
+		FAIL_TEST("Caught subclass " << e.get_type() << \
+			  " of expected " << expected_exception); \
+	    } \
+	} \
 	expected_exception = NULL;\
     } while (0)
 
